------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with OCI.Thread;
with System;
with Ada.Unchecked_Deallocation;
--with Text_IO; use Text_IO; with System.Address_Image;
package body OCI.Thick.Lobs is

   use Ada.Strings.Unbounded;
   
  type4alloc : array (Lob_Type) of UB4 := (
     File => OCI_DTYPE_FILE,
     Bin => OCI_DTYPE_LOB,
--     NChar =>OCI_DTYPE_LOB,
     Char => OCI_DTYPE_LOB
     );

   type4create_temp : array(Lob_Type) of UB1 := (
     File => 0,
     Bin  => OCI_TEMP_BLOB,
--     NChar => OCI_TEMP_NCLOB,
     Char => OCI_TEMP_CLOB
     );

   ------------------
   -- Create_Empty --
   ------------------

   function Create_Empty (Connect : Connection; LType : Lob_Type := Char) return Lob is
      use type System.Address;
      Object : Lob;
   begin
      Object.Handle := Alloc_Descriptor (
          Thread.Environment, type4alloc (Ltype));
      Object.LType := LType;
      Object.locp := new OCIHandle'(Object.Handle);
      Object.Connect := Connect;
      return Object;
   end;
   
   ----------------------
   -- Create_Temporary --
   ----------------------

   function Create_Temporary (
      Connect : Connection;
      ltype : Lob_Type := Char) return LOB
   is
      Loc : Lob := Create_Empty (Connect, LType);
   begin
      Create_Temporary (Loc);
      return Loc;
   end Create_Temporary;

   procedure Create_Temporary (Loc : in out Lob; Cache : in Boolean := True) 
   is
      rc : sword;
      To_OCI_Boolean : constant array (Boolean) of C.Int := 
         (True => Lib.True,
          False => Lib.FALSE);
   begin
      rc := OCILOBCREATETEMPORARY (svchp  => OCISvcCtx (Handle (Loc.Connect)),
                                   errhp   => Thread.Error,
                                   locp    => OCILobLocator (Loc.Handle),
                                   csid    => OCI_DEFAULT,
                                   csfrm   => SQLCS_IMPLICIT,
                                   lobtype => type4create_temp (Loc.ltype),
                                   cache    => To_OCI_Boolean (Cache),
                                   duration => OCI_DURATION_SESSION);
--     put_line("create_temp "&sword'image(rc)&' '&Address_Image(Get_Value_Internal(loc.Var.all)));
     Check_Error (rc);
   end Create_Temporary;

    procedure Create_Temporary (Stream : in out Lob_Stream; Connect : Connection) is
    begin
       Stream.Loc := Create_Empty (Connect, Stream.LType);
       Create_Temporary (Stream.Loc);
    end Create_Temporary;
    
   -------------
   -- Destroy --
   -------------

   procedure Destroy (Object : in out Lob) is
     rc : sword;
     use type System.Address;
     procedure Free is new Ada.Unchecked_Deallocation (OCIHandle, AC.Object_Pointer);
   begin
     if Object.Handle /= Empty_Handle then
--        Put_Line ("destroy " & System.Address_Image (object.locp.all));
--        Put_Line ("destroy " & System.Address_Image (AC.To_Address (object.locp)));
        rc := OCIDescriptorFree (
            descp => Object.Handle,
            dtype => type4alloc (Object.LType));
        Check_Error(rc);
     end if;
     Free (Object.locp);
   end Destroy;
   
   ----------
   -- Read --
   ----------

   procedure Read (
     loc    :  IN LOB;
     amount :  in OUT Count;
     offset : in Positive_Count;
     buffer :  System.Address) is
    amt : aliased ub4 := ub4 (amount);
    use System;
    rc : sword := OCILOBREAD (svchp => OCISvcCtx(Handle (loc.Connect)),
                        errhp => Thread.Error,
                        locp  => OCILobLocator (loc.Handle),
                        amtp  => amt'Unchecked_Access,
                        offset=> ub4 (offset),
                        bufp  => buffer,
                        bufl  => ub4 (amount),
                        ctxp  => Empty_Handle,
                        cbfp  => Empty_Handle,
                        csid  => 0);       
   begin
     pragma Assert (Loc.Handle = OCIHandle (Loc.locp.all));
--     Put_Line ("read " & Address_Image (Loc.locp.all));
--     Put_Line ("read " & Address_Image (AC.To_Address (Loc.locp)));
--     put_line ("read " & sword'image(rc));
     Check_Error (rc);
     Amount := Count(Amt);
   end Read;  

   procedure READ (
     loc    :  IN LOB;
     amount :  OUT Count;
     offset :  IN Positive_Count;
     buffer :  OUT RAW) is
     amt : Count := buffer'Length;
   begin
     Read (Loc, Amt, offset, buffer'Address);
     amount := amt;
   end read;

   procedure READ (
     loc    :  IN LOB;
     amount :  OUT Count;
     offset :  IN Positive_Count;
     buffer :  OUT String) is
     amt : Count := buffer'Length;
   begin
     Read (Loc, Amt, offset, buffer'Address);
     amount := amt;
   end read;

   -----------
   -- WRITE --
   -----------

   procedure WRITE (
     loc : IN OUT LOB;
     offset : IN Positive_Count;
     buffer : IN system.address;
     length : in ub4) is
     amt : aliased ub4 := Length;
    rc : sword := OCILOBWRITE(
                         svchp => OCISvcCtx(Handle(Loc.Connect)),
                         errhp => Thread.Error,
                         locp  => OCILobLocator(Loc.Handle),
                         amtp  => amt'Unchecked_Access,
                         offset => ub4(offset),
                         bufp   => buffer,
                         buflen => Length,
                         piece => OCI_ONE_PIECE,
                         ctxp  => Empty_Handle,
                         cbfp  => Empty_Handle,
                         csid  => 0);
     begin
       Check_Error(rc);
     end write;

   procedure WRITE (
     loc : IN OUT LOB;
     offset : IN Positive_Count;
     buffer : IN RAW) is
   begin
     write(Loc,offset,buffer'address,buffer'Length);
   end;  

   procedure WRITE (
     loc : IN OUT LOB;
     offset : IN Positive_Count;
     buffer : IN String) is
   begin
     write(Loc,offset,buffer'address,buffer'Length);
   end;  

   -------------
   -- Is_Init --
   -------------

   function Is_Init (Value : in Lob) return boolean is
      result : aliased c.int;
      rc : sword;
      use type DVoid;
   begin
   
     if Value.Handle = Empty_Handle then 
        return false;
     end if;

     rc := OCILobLocatorIsInit (envhp => Thread.Environment,
                                 errhp      => Thread.Error,
                                 locp       => OCILobLocator (Value.Handle),
                                 is_initialized => result'unchecked_access);
--     put_line("init.c.int=" & c.int'image(result));

     Check_Error(rc);
     return Boolean'Val (result);
   end Is_Init;

   -------------
   -- Is_Open --
   -------------

   function Is_Open (Value : in Lob) return boolean is
      result : aliased c.int;
      rc : sword;
      use type DVoid;
   begin
   
     if Value.Handle = Empty_Handle then 
        return false;
     end if;

     rc := OCILobIsOpen (svchp => OCISvcCtx (Handle (Value.Connect)),
                                 errhp      => Thread.Error,
                                 locp       => OCILobLocator (Value.Handle),
                                 flag       => result'unchecked_access);
--     put_line("Open.c.int=" & c.int'image(result));

     Check_Error(rc);
     return Boolean'Val (result);
   end Is_Open;

   --------------------
   -- Free_Temporary --
   --------------------

   procedure Free_Temporary (Loc : in out Lob) is
      rc : sword := OCILOBFREETEMPORARY (svchp => OCISvcCtx (Handle (Loc.Connect)),
                                   errhp => Thread.Error,
                                   locp => OCILobLocator(Loc.Handle));
   begin
--     put_line("free_temp "&sword'image(rc)&' '&Address_Image(Get_Value_Internal(loc.Var.all)));
     Check_Error(rc);
   end Free_Temporary;

    procedure Free_Temporary (Stream : in out Lob_Stream) is
    begin
       Free_Temporary (Stream.Loc);
    end Free_Temporary;
    
    procedure Reset (Stream : in out Lob_Stream) is
    begin
       Stream.Position := Positive_Count'First;
    end Reset;
    
    procedure Read (Stream : in out Lob_Stream;
	 Item   : out Stream_Element_Array;
	 Last   : out Stream_Element_Offset) is
       amount : Count;
    begin
       Read(Stream.Loc,amount => amount, 
         offset => Stream.Position,
         Buffer => Item);
       Stream.Position := Stream.Position + amount;
       Last := Stream_Element_Offset(Stream.Position);
    end Read;
    
    procedure Write(Stream : in out Lob_Stream;
       Item   : in Stream_Element_Array) is
    begin
      Write(Loc => Stream.Loc,
        Buffer => Item,
        Offset => Stream.Position);
      Stream.Position := Stream.Position+Item'Length;
    end Write;
       
    function Length (Loc : in Lob) return Count 
    is
       result : aliased ub4;
       rc : sword := OCILOBGETLENGTH(
              svchp => OCISvcCtx(Handle(Loc.Connect)),
              errhp => Thread.Error,
              locp  => OCILobLocator(Loc.Handle),
              lenp  => result'Unchecked_Access);
    begin
--      put_line("Length code="&sword'image(rc)&Ascii.Lf&
--      "Lob="&Address_Image(Get_Value_Internal(loc.Var.all))&Ascii.Lf&
--      "Connection="&Address_Image(Handle(Loc.Connect))&Ascii.Lf&
--      "Length="&ub4'Image(result));
      Check_Error(rc);
      return Count(result);
    end;
    
    function To_String (Loc : in Lob) return String is
      Len : Count := Length (Loc);
      Result : C.Char_Array(0..C.Size_T(Len-1));--String(1..Integer(Len));
    begin
      Read(Loc, Len, 1, Result'Address);
      --return Result;
      return C.To_Ada (Result,False);
    end;

    function To_Unbounded_String(Loc : in Lob) return Unbounded_String is
      Position : Count := 1;
      Buffer : String (1..16#4000#);
      Result : Unbounded_String;
      Amount : Count;
    begin
      loop
         READ (
            loc    => Loc,
            amount => Amount,
            offset =>  Position,
            buffer => buffer);
         Append (Result, Buffer(1.. Integer (Amount)));
         exit when Amount < Buffer'Length;
         Position := Position + Amount;
      end loop;
      return Result;
    end;

    function To_String(Loc : in Lob_Stream) return String is
    begin
      return To_String(Loc.Loc);
    end;
       
--    procedure Bind (Stmt : Statement; Value : in out Lob_Stream; Name : String) is
--    begin
--      Bind (Stmt, Value.Loc, Name);
--    end;
--    
--    procedure Bind (Stmt : Statement; Value : in out Lob_Stream; Position : Positive) is
--    begin
--      Bind (Stmt, Value.Loc, Position);
--    end;
--    
--    procedure Define (Stmt : Statement; Value : in out Lob_Stream; Position : Positive) is
--    begin
--      Define (Stmt, Value.Loc, Position);
--    end;
--  
--    procedure Bind (Stmt : Statement; Value : in out Lob; Name : String) is
--    begin
--      Init_Handle (Value);
--      Bind (Stmt, Value.Var.all, Name);
--      Value.Connect := Get_Connection (Stmt);
--    end;
--    
--    procedure Bind(Stmt : Statement; Value : in out Lob; Position : Positive) is
--    begin
--      Init_Handle (Value);
--      Bind (Stmt, Value.Var.all, Position);
--      Value.Connect := Get_Connection (Stmt);
--    end;
--    
--    procedure Define(Stmt : Statement; Value : in out Lob; Position : Positive) is
--    begin
--      Init_Handle (Value);
--      Define (Stmt, Value.Var.all, Position);
--      Value.Connect := Get_Connection (Stmt);
--    end;

   function Get_Lob_Type (Loc : Lob) return Lob_Type is
   begin
      return Loc.LType;
   end;

   function Get_Bind_Address (Loc : Lob) return DVoid
   is
      use type System.Address;
   begin
      pragma Assert (Loc.Handle = OCIHandle (Loc.locp.all));
--      Put_Line ("get bind address " & System.Address_Image (Loc.locp.all));
--      Put_Line ("get bind address " & System.Address_Image (AC.To_Address (Loc.locp)));
      return AC.To_Address (Loc.locp);
   end;

end OCI.Thick.Lobs;

