------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
--with Text_IO; use Text_IO;

with 
   Interfaces.C,
   Ada.Finalization;
   
package body OCI.Thread is

   Global_Env : Thread_Environment := Init_Environment;

   -----------------
   -- Environment --
   -----------------
   protected body Synch is

      function Environment return Thread_Environment is
         Envr : OCIEnv renames
            Thread.Attributes.Reference.Environment.Handle;
      begin
         if Envr = OCIEnv (Empty_Handle) then
            if Global_Env.Handle = OCIEnv(Empty_Handle) then
               Global_Env := Create;
            end if;
            Attributes.Reference.Environment := Global_Env;
-- ((Ada.Finalization.Controlled 
--               with Handle => OCIError (Empty_Handle),
--               ,
--               Last));
         end if;
         return Attributes.Reference.Environment;
      end Environment;

      function Task_Dedicated_Environment return Boolean is
      begin
         if Thread.Attributes.Reference.Environment.Handle
            /= OCIEnv (Empty_Handle) then
            return False;
         else
            Attributes.Reference.Environment := Create;
--            Attributes.Set_Value ((Ada.Finalization.Controlled 
--               with Handle => OCIError (Empty_Handle), Environment => Create));
            return True;
         end if;
      end;

   end Synch;

   function Environment return OCIEnv is
      Envr : OCIEnv renames
         Thread.Attributes.Reference.Environment.Handle;
      Env : OCIEnv := Envr;
   begin
--      Put_Line ("function Environment");
      if Env=OCIEnv(Empty_Handle) then
         return Synch.Environment.Handle;
      else
         return Env;
      end if;
   end Environment;

   -----------
   -- Error --
   -----------

   procedure Create (Error : in out Error_Type) is
      HErr : aliased OCIHandle := Empty_Handle;
      Rc : Sword;
      ERROR_HANDLE_CREATION_ERROR : exception;   
      ENVIRONMENT_SHOULD_BE_CREATED_FIRST : exception;   
      use type Interfaces.C.int;
   begin
--      Put_Line ("Create error handle");
      if Error.Environment.Handle = OCIEnv (Empty_Handle) then
         raise ENVIRONMENT_SHOULD_BE_CREATED_FIRST; 
      end if; 
      if Error.Handle /= OCIError (Empty_Handle) then
--          Put_Line ("Recreate error handle");
          RC := OCIHandleFree (OCIHandle (Error.Handle),
                OCI_HTYPE_Error);
      end if;
      if OCIHandleAlloc (Parenth => OCIHandle (Error.Environment.Handle),
         Hndlpp => herr'access,
         Htype => OCI_HTYPE_ERROR) /= OCI_SUCCESS then
         raise ERROR_HANDLE_CREATION_ERROR;
      end if;
      Error.Handle := OCIError (Herr);
   end Create;

--   procedure Initialize (Object : in out Error_Type)
--   is
--      use Ada.Finalization;
--   begin
--      Initialize (Controlled (Object));
--      Put_Line ("Initialized");
--   end;

--   procedure Adjust (Object : in out Error_Type)
--   is
--      use Ada.Finalization;
--   begin
--      Adjust (Controlled (Object));
--      Put_Line ("Adjusted");
--   end;
   
   procedure Finalize (Object : in out Error_Type) is
      Rc : SWord;
   begin
     if Object.Handle /= OCIError (Empty_Handle) then
        RC := OCIHandleFree (OCIHandle (Object.Handle),
                OCI_HTYPE_Error);
--        Put_Line ("Final " & SWord'Image (Rc));
     end if;
   end Finalize;

   function Error return OCIError is
      Errr : OCIError renames Attributes.Reference.Handle;
      Err : OCIError := Errr;
      Dummy : OCIEnv;
   begin
--      Put_Line ("function Error");
      if Err = OCIError(Empty_Handle) then
--         Put_Line ("Need error handle");
         Dummy := Environment;
         Create (Attributes.Reference.all);
         return Errr;
      else
--         Put_Line ("func error done;");
         return Err;
      end if;
   end Error;


  -- initalize from Extproc Context
  procedure Set_Error (Env : OCIEnv; Err : OCIError)
  is
  begin
      Attributes.Reference.Environment := (RF.Controlled_Reference with Handle => Env);
      Attributes.Reference.Handle := Err;
--     Attributes.Set_Value ( (Ada.Finalization.Controlled with 
--          Handle => Err,
--          Environment => (RF.Controlled_Reference with Handle => Env)));
  end;

end OCI.Thread;

