------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with
   Text_IO,
   Oci.Thick.Connections,
   Oci.Thick.Statements,
   Oci.Thick.Binds.Lob,
   Oci.Thick.Lobs;

with Ada.Exceptions;
with GNAT.Traceback.Symbolic;

procedure Test_Lob0 is
   use
      Text_IO,
      Oci.Thick;
      
   use
      Connections,
      Statements,
      Binds.Lob,
      Lobs;

   Connect : Connection := Logon (
        "scott/tiger");

   Loc : Lob := Create_Empty (Connect);
   Bind : Bind_Lob;
   Buffout : String (1 .. 79) := (others => '_');
   Stmt : Statement;
   offset : Lobs.Count := 1;  
   amt : Lobs.Count := Buffout'Length;  
   sum_amt : Lobs.Count := 0;  

   procedure Pause (msg : String)
   is
      Char : Character;
   begin
      Put_Line (msg);
      Get_Immediate (Char);
   end Pause;



begin
   Create_Temporary (Loc, False);

   Stmt := Prepare (Connect,
      "begin "&
      "  for i in 1..7000 loop "&
      "     DBMS_LOB.WRITEAPPEND (:lob, 4, lpad(i,4));"&
      "  end loop; "&
      "end;");

   Put_Line("Prepared");
   
   Binds.Lob.Bind (Stmt, Bind, Loc, "lob");

   Put_Line("Binded");
   
   if Is_Init (Loc) then
      Put_Line("init");
   else
      Put_Line("not initialized");
   end if;
   
  -- Pause ("press a key for execute");
   
   Execute (Stmt);
   Put_Line("Executed");

   if Is_Init (Loc) then
      Put_Line("init");
   else
      Put_Line("not initialized");
   end if;
   
   loop
     Read (Loc, amt, offset, Buffout);
     Put_Line (Buffout);
     Buffout := (others => buffout (buffout'First));
--     Write(Loc,offset,Buffout);
     offset := offset+amt+1;
     sum_amt := sum_amt+amt;
--     exit;
     exit when amt < Buffout'Length;
   end loop;
   Put_Line("readed " & Lobs.Count'Image (sum_amt) & Boolean'Image (Is_Init (Loc)));

   Pause ("press a key for free temporary");

   Free_Temporary (Loc);

  -- Pause ("press a key for exit.");

   Put_Line("end "&Boolean'Image(Is_Init(Loc)));
exception
      when e: others =>
      
         Put_Line (Standard_Error, Ada.Exceptions.Exception_Information (E));
         Put_Line (Standard_Error, GNAT.Traceback.Symbolic.Symbolic_Traceback (E));

end;